@@ -1,5 +1,5 @@
 use proc_macro::{self, TokenStream};
-use quote::quote;
+use quote::{quote, format_ident};
 use syn::{parse_macro_input, DeriveInput, FieldsNamed};
 
 #[proc_macro_derive(FieldAccessor)]
@@ -9,11 +9,17 @@ pub fn get(input: TokenStream) -> TokenStream {
     let output = match data {
         syn::Data::Struct(s) => match s.fields {
             syn::Fields::Named(FieldsNamed { named, .. }) => {
+                let idents_enum = named.iter().map(|f| &f.ident);
+                let idents2_getenum = named.iter().map(|f| &f.ident);
+                let tys = named.iter().map(|f| &f.ty).clone();
+                let enumname = format_ident!("{}{}", ident, "FieldEnum");
+
                 let newnamed = named.clone();
                 let mut set_quotes = vec![];
                 let mut get_quotes = vec![];
                 let mut set_tys = vec![];
                 let mut get_tys = vec![];
+
                 for name in newnamed.clone().iter() {
                     if get_tys.contains(&name.ty) {
                     } else {
@@ -44,6 +50,12 @@ pub fn get(input: TokenStream) -> TokenStream {
                     }
                 }
                 quote! {
+
+                    #[derive(Debug, PartialEq, PartialOrd, Clone)]
+                    enum #enumname{
+                        #(#idents_enum(#tys)),*
+                    }
+
                     trait GetterSetter<T> {
                         fn get(&mut self, field_string: &String) -> Result<&T, String>;
                         fn set(&mut self, field_string: &String, value: T) -> Result<(), String>;
@@ -64,6 +76,16 @@ pub fn get(input: TokenStream) -> TokenStream {
                             }
                         }
                     )*
+                    impl #ident {
+                        fn getenum(&mut self, field_string: &String) -> Result<#enumname, String> {
+                            match &**field_string {
+                                #(stringify!(#idents2_getenum) => {
+                                    Ok(#enumname::#idents2_getenum(self.#idents2_getenum.clone()))
+                                }),*
+                                _ => Err(format!("invalid field name to get '{}'", field_string)),
+                            }
+                        }
+                    }
                 }
             }
             _ => panic!("unsupported fields"),
